import time
from django import forms
from django.conf import settings
from django.contrib.comments.models import Comment
from django.contrib.comments.forms import COMMENT_MAX_LENGTH
from django.utils.text import get_text_list
from django.utils.translation import ungettext, ugettext, ugettext_lazy as _
from django.forms.util import ErrorDict
from django.utils.crypto import salted_hmac, constant_time_compare


class CommentEditForm(forms.ModelForm):
    """
    ModelForm for editing existing comments.
    Quacks like the CommentSecurityForm in django.contrib.comments.forms
    """
    def __init__(self, *args, **kwargs):
        super(CommentEditForm, self).__init__(*args, **kwargs)
        
        # initiate the form with security data
        self.initial.update(self.generate_security_data())
        
        self.fields["user_name"].widget.attrs["readonly"] = "readonly"
        self.fields["user_email"].widget.attrs["readonly"] = "readonly"
        self.fields["user_url"].widget.attrs["placeholder"] = "Homepage"
        
    user_name = forms.CharField(label=_("Name"), max_length=50)
    user_email = forms.EmailField(label=_("Email address"))
    user_url = forms.URLField(label=_("URL"), required=False)
    comment = forms.CharField(label=_("Comment"), widget=forms.Textarea,
                                    max_length=COMMENT_MAX_LENGTH)
    
    # Security fields
    timestamp = forms.IntegerField(widget=forms.HiddenInput)
    security_hash = forms.CharField(min_length=40, max_length=40, widget=forms.HiddenInput)
    honeypot = forms.CharField(required=False, label=_("If you enter anything in this field "\
                                                       "your comment will be treated as spam."))
    
    class Meta:
        model = Comment
        fields = ("user_name", "user_email", "user_url", "comment",
                  "timestamp", "security_hash", "honeypot")

    def security_errors(self):
        """
        Return the errors associated with security
        """
        errors = ErrorDict()
        for f in ["honeypot", "timestamp", "security_hash"]:
            if f in self.errors:
                errors[f] = self.errors[f]
        return errors
    
    def generate_security_data(self):
        """
        Generate initial security data
        """
        # Use the original timestamp
        timestamp = str(int(time.mktime(self.instance.submit_date.timetuple())))
        security_dict = {
            "content_type": str(self.instance.content_type.pk),
            "object_pk": str(self.instance.pk),
            "timestamp": timestamp,
            "security_hash": self.initial_security_hash(timestamp)
        }
        return security_dict
    
    def initial_security_hash(self, timestamp):
        """
        Generate the initial security hash from self.content_object
        and a (unix) timestamp.
        """
        initial_security_dict = {
            "content_type": str(self.instance.content_type.pk),
            "object_pk": str(self.instance.pk),
            "timestamp": timestamp
        }
        return self.generate_security_hash(**initial_security_dict)
    
    def generate_security_hash(self, content_type, object_pk, timestamp):
        """
        Generate a HMAC security hash from the provided info.
        """
        info = (content_type, object_pk, timestamp)
        key_salt = "comments_extension.forms.CommentEditForm"
        value = "-".join(info)
        return salted_hmac(key_salt, value).hexdigest()
        
    #
    # Clean methods
    #
    def clean_security_hash(self):
        """
        Make sure the security hash match
        """
        timestamp = str(int(time.mktime(self.instance.submit_date.timetuple())))
        security_hash_dict = {
            "content_type": self.data.get("content_type", str(self.instance.content_type.pk)),
            "object_pk": self.data.get("object_pk", str(self.instance.pk)),
            "timestamp": self.data.get("timestamp", timestamp)
        }
        expected_hash = self.generate_security_hash(**security_hash_dict)
        actual_hash = self.cleaned_data["security_hash"]
        if not constant_time_compare(expected_hash, actual_hash):
            raise forms.ValidationError("Security hash check failed.")
        return actual_hash
    
    def clean_comment(self):
        """
        If COMMENTS_ALLOW_PROFANITIES is False, check that the comment doesn't
        contain anything in PROFANITIES_LIST.
        """
        comment = self.cleaned_data["comment"]
        if settings.COMMENTS_ALLOW_PROFANITIES == False:
            bad_words = [w for w in settings.PROFANITIES_LIST if w in comment.lower()]
            if bad_words:
                raise forms.ValidationError(ungettext(
                    "Watch your mouth! The word %s is not allowed here.",
                    "Watch your mouth! The words %s are not allowed here.",
                    len(bad_words)) % get_text_list(
                        ["'%s%s%s'" % (i[0], "-" * (len(i) - 2), i[-1])
                         for i in bad_words], ugettext("and")))
        return comment
    
    def clean_honeypot(self):
        """
        Check that honeypot field is empty
        """
        value = self.cleaned_data["honeypot"]
        if value:
            raise forms.ValidationError(self.fields["honeypot"].label)
        return value



from __future__ import absolute_import
from django.core import urlresolvers
from django.contrib.comments import get_comment_app, get_comment_app_name,\
    DEFAULT_COMMENTS_APP
from comments_extension.forms import CommentEditForm


def get_edit_form():
    """
    Returns a (new) comment edit form object
    """
    if get_comment_app_name() != DEFAULT_COMMENTS_APP and hasattr(get_comment_app(), "get_edit_form"):
        return get_comment_app().get_edit_form()
    else:
        return CommentEditForm
    
    
def get_edit_modelform(comment):
    """
    Returns the comment ModelForm instance
    """
    if get_comment_app_name() != DEFAULT_COMMENTS_APP and hasattr(get_comment_app(), "get_edit_modelform"):
        return get_comment_app().get_edit_modelform()
    else:
        return CommentEditForm(instance=comment)


def get_edit_form_target(comment):
    """
    Returns the target URL for the comment edit form submission view.
    """
    if get_comment_app_name() != DEFAULT_COMMENTS_APP and hasattr(get_comment_app(), "get_edit_form_target"):
        return get_comment_app().get_edit_form_target()
    else:
        return urlresolvers.reverse("comments_extension.views.moderation.edit", args=(comment.id,))



from __future__ import absolute_import
from django import template
from django.template.loader import render_to_string
from django.contrib.comments.templatetags.comments import CommentFormNode,\
    BaseCommentNode
import comments_extension


register = template.Library()


class CommentEditFormNode(CommentFormNode):
    """
    Insert a form for the comment model into the context.
    """
    def get_form(self, context):
        obj = self.get_object(context)
        if obj:
            return comments_extension.get_edit_modelform(obj)
        else:
            return None
        

class RenderCommentEditFormNode(CommentFormNode):
    """
    Class method to parse render_comment_edit_form and return a Node
    prefilled with existing data.
    """
    
    @classmethod
    def handle_token(cls, parser, token):
        """Class method to parse render_comment_form and return a Node."""
        tokens = token.contents.split()
        if tokens[1] != 'for':
            raise template.TemplateSyntaxError("Second argument in %r tag must be 'for'" % tokens[0])

        # {% render_comment_form for obj %}
        if len(tokens) == 3:
            return cls(object_expr=parser.compile_filter(tokens[2]))

        # {% render_comment_form for app.models pk %}
        elif len(tokens) == 4:
            return cls(
                ctype = BaseCommentNode.lookup_content_type(tokens[2], tokens[0]),
                object_pk_expr = parser.compile_filter(tokens[3])
            )
    
    def get_target_ctype_pk(self, context):
        try:
            obj = self.object_expr.resolve(context)
        except template.VariableDoesNotExist:
            return None, None
        return obj.content_type, obj.pk
    
    def render(self, context):
        ctype, object_pk = self.get_target_ctype_pk(context)
        if object_pk:
            template_search_list = [
                "comments/%s/%s/edit.html" % (ctype.app_label, ctype.model),
                "comments/%s/edit.html" % ctype.model,
                "comments/edit.html"
            ]
            context.push()
            form = comments_extension.get_edit_modelform(self.get_object(context))
            formstr = render_to_string(template_search_list, {"form": form}, context)
            context.pop()
            return formstr
        else:
            return ""
        

@register.tag
def get_comment_edit_form(parser, token):
    """
    Get a form object to edit existing comment.
    
    Syntax::
    
        {% get_comment_edit_form for [object] as [varname] %}
        {% get_comment_edit_form for [app].[model] [object_id] as [varname] %}
    """
    return CommentEditFormNode.handle_token(parser, token)


@register.tag
def render_comment_edit_form(parser, token):
    """
    Render the comment edit form (as returned by ``{% render_comment_edit_form %}``)
    through the ``comments/edit.html`` template.

    Syntax::

        {% render_comment_edit_form for [object] %}
        {% render_comment_edit_form for [app].[model] [object_id] %}
    """
    return RenderCommentEditFormNode.handle_token(parser, token)


@register.simple_tag
def comment_edit_form_target(comment):
    """
    Get the target URL for the comment form.

    Example::

        <form action="{% comment_edit_form_target comment %}" method="post">
    """
    return comments_extension.get_edit_form_target(comment)



from django.conf.urls import patterns, url

urlpatterns = patterns("comments_extension.views",
    url(r"^edit/(\d+)/$", view="moderation.edit", name="comments-edit"),
    url(r"^edited/$", view="moderation.edit_done", name="comments-edit-done"),
)



from __future__ import absolute_import

from django.conf import settings
from django.http import HttpResponse, HttpResponseBadRequest
from django.utils.html import escape
from django.contrib import comments
from django.contrib.comments import signals
from django.contrib.comments.views.utils import confirmation_view, next_redirect
from django.contrib.auth.decorators import user_passes_test
from django.shortcuts import get_object_or_404, render_to_response
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.http import require_POST
from django.template.loader import render_to_string
from django.template.context import RequestContext

import comments_extension


class CommentEditBadRequest(HttpResponseBadRequest):
    """
    Response returned when a comment edit is invalid. If ``DEBUG`` is on a
    nice-ish error message will be displayed (for debugging purposes), but in
    production mode a simple opaque 400 page will be displayed.
    """
    def __init__(self, why):
        super(CommentEditBadRequest, self).__init__()
        if settings.DEBUG:
            self.content = render_to_string("comments/400-edit-debug.html", {"why": why})


@csrf_protect
@require_POST
@user_passes_test(lambda u: u.has_perm("comments.change_comment")
                  or u.has_perm("comment.can_moderate"))
def edit(request, comment_id, next=None):
    """
    Edit a comment.
    
    Requires HTTP POST and "can change comments" or "can moderate comments",
    permission. Users can also only edit comments they own, unless they are
    given "comments.change_comment" on this specific comment object.
    
    If ``POST['submit'] == "preview"`` or there are errors,
    a preview template ``comments/preview.html`` will be rendered.
    
    Templates: `comments/edit.html`,
    Context:
        comment
            the `comments.comment` object to be edited.
    """
    comment = get_object_or_404(comments.get_model(), pk=comment_id, site__pk=settings.SITE_ID)
    
    # Make sure user has correct permissions to change the comment,
    # or return a 401 Unauthorized error.
    if not (request.user == comment.user or request.user.has_perms(["comments.change_comment"], comment)):
        return HttpResponse("Unauthorized", status=401)
    
    # Populate POST data with all required initial data
    # unless they are already in POST
    data = request.POST.copy()
    if not data.get("user_name", ""):
        data["user_name"] = request.user.get_full_name() or request.user.username
    if not data.get("user_email"):
        data["user_email"] = request.user.email
    
    next = data.get("next", next)
    CommentEditForm = comments_extension.get_edit_form()
    form = CommentEditForm(data, instance=comment)
    
    if form.security_errors():
        # NOTE: security hash fails!
        return CommentEditBadRequest(
            "The comment form failed security verification: %s" % \
                escape(str(form.security_errors())))
    
    # If there are errors, or if a preview is requested
    if form.errors or "preview" in data:
        app_label, model = (form.instance.content_type.app_label, form.instance.content_type.model)
        template_search_list = [
            "comments/%s/%s/edit-preview.html" % (app_label, model),
            "comments/%s/edit-preview.html" % model,
            "comments/edit-preview.html"
        ]
        return render_to_response(
            template_search_list, {
                "comment_obj": comment,
                "comment": form.data.get("comment", ""),
                "form": form,
                "next": next,
            },
            RequestContext(request, {})
        )
        
    # Otherwise, try to save the comment and emit signals
    if form.is_valid():
        MODERATOR_EDITED = "moderator edited"
        flag, created = comments.models.CommentFlag.objects.get_or_create(
            comment = form.instance,
            user = request.user,
            flag = MODERATOR_EDITED
        )
        
        form.instance.is_removed = False
        form.save()
        
        signals.comment_was_flagged.send(
            sender = comment.__class__,
            comment = comment,
            flag = flag,
            created = created,
            request = request
        )
        
        return next_redirect(request.POST.copy(), next, edit_done, c=comment.pk)
    
    else:
        # If we got here, raise Bad Request error.
        return CommentEditBadRequest("Could not complete request!")
        

edit_done = confirmation_view(
    template = "comments/edited.html",
    doc = 'Displays a "comment was edited" success page.'
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distutils.core import setup

setup(
    name = "django-comments-extension",
    version = "1.1",
    description = "contrib.comments extensions.",
    long_description = "Provides edit functionality to the contrib.comments framework",
    keywords = "django edit comments",
    license = open("LICENSE.md").read(),
    author = "Rolf Håvard Blindheim",
    author_email = "rhblind@gmail.com",
    url = "https://github.com/rhblind/django-comments-extension",
    packages = [
        "comments_extension",
        "comments_extension.views",
        "comments_extension.templatetags"
    ],
    package_data = {
        "comments_extension": [
            "templates/comments/*.html",
        ]
    },
    classifiers = [
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
